package de.bsvrz.buv.plugin.darstellung.actions;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.Date;

import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PlatformUI;

import de.bsvrz.buv.plugin.dobj.internal.DObjPlugin;
import de.bsvrz.buv.plugin.dobj.util.IDavZeitStempelManager;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Ein Status-Item welches den letzten empfangenen Datenzeitstempel anzeigt.
 * 
 * @author BitCtrl Systems GmbH, bitcontrol
 * @version $Id$
 */
public class LetzterEmpfangenerDavZeitStempelStatus extends DarstellungStatus
		implements IDavZeitStempelManager {

	private static final SimpleDateFormat FORMAT = new SimpleDateFormat(
			"HH:mm:ss");

	private long letzterEmpfangenerZeitStempel = -1;

	/**
	 * Initialisiert das Status Item.
	 * 
	 * @param page
	 *            die Workbench Page fuer den das Item bestimmt ist.
	 */
	public LetzterEmpfangenerDavZeitStempelStatus(final IWorkbenchPage page) {
		super(page, LetzterEmpfangenerDavZeitStempelStatus.class.toString(), 10);
	}

	@Override
	public void setLetztenEmpfangenenZeitStempel(final long zeitStempel,
			final SystemObject obj) {
		if (zeitStempel == -1) {
			final StringWriter sw = new StringWriter();
			final PrintWriter pw = new PrintWriter(sw, true);
			new Exception().printStackTrace(pw);
			pw.flush();
			sw.flush();
			final String errorStackTrace = sw.toString();
			PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

				@Override
				public void run() {
					setText("Err: Log!");
					DObjPlugin
							.getDefault()
							.getLogger()
							.error("Zeitstempelproblem1 in " + obj + ":\n"
									+ errorStackTrace);
				}

			});
		}
		if (zeitStempel > letzterEmpfangenerZeitStempel) {
			letzterEmpfangenerZeitStempel = zeitStempel;
			DObjPlugin.getDefault().getLogger()
					.error("Zeitstempelproblem2 in " + obj);
			PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {

				@Override
				public void run() {
					setText(FORMAT.format(new Date(zeitStempel)));
				}

			});
		}
	}

	@Override
	protected void activate() {
		if (getViewer().getProperty(IDavZeitStempelManager.class.toString()) == null) {
			getViewer().setProperty(IDavZeitStempelManager.class.toString(),
					this);
		}
	}

	@Override
	protected void deactivate() {
		//
	}

}
