package de.bsvrz.buv.plugin.darstellung.actions;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;

import com.bitctrl.lib.eclipse.emf.util.EmfUtil;

import de.bsvrz.buv.plugin.darstellung.editors.AnsichtEditor;
import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.dobj.internal.DObjPlugin;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.bitctrl.eclipse.editors.EmfEinstellungenEditorInput;

/**
 * Aktion zum ffnen einer Ansicht.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class ZeigeAnsichtAction extends Action {

	private final IWorkbenchPart part;

	/**
	 * Initialisiert die Aktion.
	 * 
	 * @param part
	 *            der Workbench Part an dem die Aktion gebunden ist.
	 */
	public ZeigeAnsichtAction(final IWorkbenchPart part) {
		super("Ansicht ffnen");
		setId(DarstellungActionConstants.ZEIGE_ANSICHT);
		setActionDefinitionId(DarstellungActionConstants.ZEIGE_ANSICHT);

		this.part = part;
	}

	@Override
	public void run() {
		final ITreeSelection selection = (ITreeSelection) part.getSite()
				.getSelectionProvider().getSelection();
		for (final TreePath path : selection.getPaths()) {
			final EinstellungsArt einstellungsArt = (EinstellungsArt) path
					.getFirstSegment();
			if (path.getLastSegment() instanceof Ansicht) {
				final Ansicht ansicht = (Ansicht) path.getLastSegment();

				final IStatus status = EmfUtil.validate(ansicht,
						DObjPlugin.PLUGIN_ID);
				if (status.isOK()) {
					final EmfEinstellungenEditorInput input = new EmfEinstellungenEditorInput(
							ansicht, einstellungsArt);
					input.setExists(true);

					try {
						PlatformUI.getWorkbench().getActiveWorkbenchWindow()
								.getActivePage()
								.openEditor(input, AnsichtEditor.EDITOR_ID);
					} catch (final PartInitException ex) {
						DObjPlugin
								.getDefault()
								.getLogger()
								.error("Die Ansicht \"" + ansicht
										+ "\" konnte nicht geffnet werden.",
										ex);
					}

				}

				ErrorDialog
						.openError(
								part.getSite().getShell(),
								"Ansicht nicht darstellbar",
								"Die Ansicht \""
										+ ansicht.getName()
										+ "\" kann nicht geffnet werden, weil sie nicht lesbar ist.",
								status);
			}
		}
	}

}
