package de.bsvrz.buv.plugin.darstellung.commands;

import org.eclipse.core.runtime.Assert;
import org.eclipse.gef.commands.Command;

import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.dobj.model.DoModel;

/**
 * Verschiebt ein Darstellungsobjekt von einer Ebene auf eine andere.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class AddDoModelToEbeneCommand extends Command {

	private final Ebene ebene;
	private final DoModel model;
	private Ebene oldEbene;
	private int indexInOldEbene;

	public AddDoModelToEbeneCommand(final DoModel model, final Ebene ebene) {
		Assert.isNotNull(model, "model");
		Assert.isLegal(model.eContainer() instanceof Ebene,
				"model muss in einer Ebene liegen");
		Assert.isNotNull(ebene, "ebene");

		this.model = model;
		this.ebene = ebene;

		setLabel("Darstellungsobjekt auf andere Ebene verschieben");
	}

	@Override
	public void execute() {
		oldEbene = (Ebene) model.eContainer();
		indexInOldEbene = oldEbene.getDoObjekte().indexOf(model);

		redo();
	}

	@Override
	public void redo() {
		ebene.getDoObjekte().add(model);
	}

	@Override
	public void undo() {
		oldEbene.getDoObjekte().add(indexInOldEbene, model);
	}

}
