package de.bsvrz.buv.plugin.darstellung.commands;

import org.eclipse.core.runtime.Assert;
import org.eclipse.gef.commands.Command;

import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;

/**
 * Legt eine neue Ebene in einer Darstellung an. Die erste Ebene heit einfach
 * "Ebene" jede weitere wird durchnummeriert.
 * 
 * <p>
 * Eine Darstellung muss mindestens eine Ebene besitze, also kann eine Ebene nur
 * gelscht werden, wenn es noch mindestens eine andere gibt.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class CreateEbeneCommand extends Command {

	private final Darstellung darstellung;
	private Ebene ebene;

	public CreateEbeneCommand(final Darstellung darstellung) {
		Assert.isNotNull(darstellung, "darstellung");

		this.darstellung = darstellung;

		setLabel("Neue Ebene anlegen");
	}

	@Override
	public void execute() {
		ebene = createEbene();
		redo();
	}

	private Ebene createEbene() {
		final Ebene e = darstellung.createEbene();
		String name = "Ebene";
		if (!darstellung.getEbenen().isEmpty()) {
			name = (darstellung.getEbenen().size() + 1) + ". " + name;
		}
		e.setName(name);
		return e;
	}

	@Override
	public void redo() {
		darstellung.getEbenen().add(ebene);
	}

	@Override
	public boolean canUndo() {
		return darstellung.getEbenen().size() > 1;
	}

	@Override
	public void undo() {
		darstellung.getEbenen().remove(ebene);
	}

}
