package de.bsvrz.buv.plugin.darstellung.commands;

import org.eclipse.core.runtime.Assert;
import org.eclipse.gef.commands.Command;

import de.bsvrz.buv.plugin.darstellung.model.AutoEbene;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;

/**
 * Entfernt einen Darstellungsobjekttyp aus einer Autoebene.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class RemoveDoTypFromAutoEbeneCommand extends Command {

	private final AutoEbene ebene;
	private final DoTyp doTyp;
	private int doTypIndexInEbene;
	private boolean removeDoTypFromDarstellung;

	public RemoveDoTypFromAutoEbeneCommand(final DoTyp doTyp,
			final AutoEbene ebene) {
		Assert.isNotNull(doTyp, "doTyp");
		Assert.isNotNull(ebene, "ebene");

		this.doTyp = doTyp;
		this.ebene = ebene;

		setLabel("Darstellungsobjekttyp aus Autoebene entfernen");
	}

	@Override
	public void execute() {
		removeDoTypFromDarstellung = !isNeededDoTyp();
		doTypIndexInEbene = getIndexInEbene(ebene);

		redo();
	}

	private int getIndexInEbene(final AutoEbene e) {
		return e.getDoTypen().indexOf(doTyp);
	}

	private boolean isNeededDoTyp() {
		return CommandUtil.isUsedDoTypInDarstellung(doTyp,
				ebene.getDarstellung(), ebene);
	}

	@Override
	public void redo() {
		ebene.getDoTypen().remove(doTypIndexInEbene);

		if (removeDoTypFromDarstellung) {
			ebene.getDarstellung().getDoTypen().remove(doTyp);
		}
	}

	@Override
	public void undo() {
		ebene.getDoTypen().add(doTypIndexInEbene, doTyp);

		if (removeDoTypFromDarstellung) {
			ebene.getDarstellung().getDoTypen().add(doTyp);
		}
	}

}
