package de.bsvrz.buv.plugin.darstellung.editparts.tree;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.RootEditPart;
import org.eclipse.gef.editpolicies.RootComponentEditPolicy;

import com.bitctrl.lib.eclipse.emf.gef.model.EListNode;

import de.bsvrz.buv.plugin.darstellung.model.Darstellung;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungPackage;
import de.bsvrz.buv.plugin.darstellung.model.MassstaeblicheDarstellung;
import de.bsvrz.buv.plugin.darstellung.model.StilisierteDarstellung;
import de.bsvrz.buv.plugin.dobj.editparts.tree.BaseTreeEditPart;

/**
 * Tree Edit Part fr eine {@link Darstellung}.
 * 
 * <p>
 * Untersttzt die {@link MassstaeblicheDarstellung} und
 * {@link StilisierteDarstellung}.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class DarstellungTreeEditPart extends BaseTreeEditPart<Darstellung> {

	private EListNode ebenen;
	private EListNode spalten;
	private EListNode doTypen;

	@Override
	public void setModel(final Object model) {
		super.setModel(model);

		ebenen = null;
		spalten = null;
		doTypen = null;
	}

	/**
	 * Gibt Zwischenknoten fr die Ebenen, die Darstellungsspalten (nur
	 * stilisierte Darstellung) und die in der Darstellung verwendeten
	 * Darstellungsobjektypen zurck.
	 * 
	 * <p>
	 * Der Zwischenknoten fr die Darstellungsobjekttypen wird nur
	 * zurckgegeben, wenn die Darstellung bearbeitet wird. Das ist z.&nbsp;B.
	 * im Vorpositionierer der Fall.
	 */
	@Override
	protected List<?> getModelChildren() {
		if (getParent() instanceof RootEditPart) {
			return Collections.singletonList(getModel());
		}

		final List<EListNode> result = new ArrayList<EListNode>();

		if (ebenen == null) {
			ebenen = new EListNode(getModel(),
					DarstellungPackage.Literals.DARSTELLUNG__EBENEN);
		}
		result.add(ebenen);

		if (getModel() instanceof StilisierteDarstellung) {
			if (spalten == null) {
				spalten = new EListNode(
						getModel(),
						DarstellungPackage.Literals.STILISIERTE_DARSTELLUNG__SPALTEN);
			}
			result.add(spalten);
		}

		if (isEditor()) {
			if (doTypen == null) {
				doTypen = new EListNode(getModel(),
						DarstellungPackage.Literals.DARSTELLUNG__DO_TYPEN);
			}
			result.add(doTypen);
		}

		return result;
	}

	@Override
	protected void createEditPolicies() {
		super.createEditPolicies();

		if (getParent() instanceof RootEditPart) {
			installEditPolicy(EditPolicy.COMPONENT_ROLE,
					new RootComponentEditPolicy());
		}
	}

	@Override
	public void notifyChanged(final Notification notification) {
		final int type = notification.getEventType();
		final int featureID = notification.getFeatureID(Darstellung.class);

		final int size = getChildren().size();
		Map<Object, EditPart> modelToEditPart = Collections.emptyMap();
		if (size > 0) {
			modelToEditPart = new HashMap<Object, EditPart>(size);
			for (int i = 0; i < size; i++) {
				final EditPart editPart = (EditPart) children.get(i);
				modelToEditPart.put(editPart.getModel(), editPart);
			}
		}

		switch (type) {
		case Notification.SET:
			switch (featureID) {
			case DarstellungPackage.DARSTELLUNG__NAME:
				refreshVisuals();
				break;
			}
			break;
		case Notification.ADD:
		case Notification.ADD_MANY:
		case Notification.REMOVE:
		case Notification.REMOVE_MANY:
		case Notification.MOVE:
			switch (featureID) {
			case DarstellungPackage.DARSTELLUNG__EBENEN:
				if (modelToEditPart.get(ebenen) != null) {
					modelToEditPart.get(ebenen).refresh();
				}
				break;
			case DarstellungPackage.STILISIERTE_DARSTELLUNG__SPALTEN:
				if (modelToEditPart.get(spalten) != null) {
					modelToEditPart.get(spalten).refresh();
				}
				break;
			case DarstellungPackage.DARSTELLUNG__DO_TYPEN:
				if (modelToEditPart.get(doTypen) != null) {
					modelToEditPart.get(doTypen).refresh();
				}
				break;
			}
			break;
		}

	}

}
