package de.bsvrz.buv.plugin.darstellung.editpolicies;

import org.eclipse.core.runtime.Platform;
import org.eclipse.draw2d.geometry.Dimension;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.gef.editpolicies.LayoutEditPolicy;
import org.eclipse.gef.editpolicies.NonResizableEditPolicy;
import org.eclipse.gef.editpolicies.ResizableEditPolicy;
import org.eclipse.gef.editpolicies.XYLayoutEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.CreateRequest;

import com.bitctrl.lib.eclipse.emf.gef.model.Sized;

import de.bsvrz.buv.plugin.darstellung.commands.AddDoModelToDarstellungsSpalteCommand;
import de.bsvrz.buv.plugin.darstellung.commands.CreateDoModelCommand;
import de.bsvrz.buv.plugin.darstellung.commands.MoveDoModelCommand;
import de.bsvrz.buv.plugin.darstellung.commands.ResizeDoModelCommand;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungsSpalte;
import de.bsvrz.buv.plugin.darstellung.model.Ebene;
import de.bsvrz.buv.plugin.dobj.HotspotConverter;
import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.model.DoTyp;

/**
 * {@link LayoutEditPolicy} fr eine {@link DarstellungsSpalte}.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class DarstellungsSpalteXYLayoutEditPolicy extends XYLayoutEditPolicy {

	/**
	 * Erzeugt ein entsprechendes Command, wenn ein {@link DoModel} verschoben
	 * oder in der Gre gendert werden soll.
	 */
	@Override
	protected Command createChangeConstraintCommand(
			final ChangeBoundsRequest request, final EditPart child,
			final Object constraint) {
		if (child.getModel() instanceof DoModel
				&& constraint instanceof Rectangle) {
			final DoModel model = (DoModel) child.getModel();
			final Rectangle bounds = (Rectangle) constraint;
			final Point location = getHotspotLocation(child, bounds);

			final CompoundCommand command = new CompoundCommand(
					"Darstellungsobjekt Verschieben und Skalieren");
			command.add(new MoveDoModelCommand(model, location));

			if (model instanceof Sized) {
				command.add(new ResizeDoModelCommand(model, bounds.getSize()));
			}

			return command.unwrap();
		}

		// Fallback
		return null;
	}

	private Point getHotspotLocation(final EditPart child,
			final Rectangle bounds) {
		HotspotConverter hotspotConverter = (HotspotConverter) child
				.getAdapter(HotspotConverter.class);
		if (hotspotConverter == null) {
			hotspotConverter = (HotspotConverter) Platform.getAdapterManager()
					.getAdapter(child, HotspotConverter.class);
		}

		Point location;
		if (hotspotConverter != null) {
			location = hotspotConverter
					.convertViewToModel(bounds.getLocation());
		} else {
			location = bounds.getLocation();
		}
		return location;
	}

	/**
	 * Gibt immer <code>null</code> zurck.
	 */
	@Override
	protected Command createChangeConstraintCommand(final EditPart child,
			final Object constraint) {
		// Wird vom GEF nicht mehr verwendet.
		return null;
	}

	/**
	 * Erzeugt ein entsprechendes Command, wenn ein Objekt vom Typ {@link DoTyp}
	 * angelegt werden soll.
	 */
	@Override
	protected Command getCreateCommand(final CreateRequest request) {
		final Object childClass = request.getNewObjectType();
		if (childClass instanceof DoTyp) {
			final Ebene ebene = (Ebene) getHost().getParent().getModel();
			final DarstellungsSpalte spalte = (DarstellungsSpalte) getHost()
					.getModel();
			final DoModel model = (DoModel) request.getNewObject();
			final Rectangle constraint = (Rectangle) getConstraintFor(request);
			final CreateDoModelCommand cmd = new CreateDoModelCommand(model,
					ebene, spalte);
			cmd.setLocation(constraint.getLocation());
			if (model instanceof Sized) {
				final Dimension size = constraint.getSize();
				if (!size.isEmpty()) {
					cmd.setSize(size);
				}
			}
			return cmd;
		}

		// Fallback
		return null;
	}

	@Override
	protected Command createAddCommand(final EditPart child,
			final Object constraint) {
		if (child.getModel() instanceof DoModel
				&& constraint instanceof Rectangle) {
			final DarstellungsSpalte spalte = (DarstellungsSpalte) getHost()
					.getModel();
			final DoModel model = (DoModel) child.getModel();
			final Rectangle bounds = (Rectangle) constraint;
			final Point location = getHotspotLocation(child, bounds);
			return new AddDoModelToDarstellungsSpalteCommand(model, location,
					spalte);
		}

		// fallback
		return null;
	}

	/**
	 * Installiert fr EditParts die ein {@link DoModel} als Modell haben
	 * zustzlich eine {@link DoModelEditPolicy}.
	 * 
	 * @return {@link NonResizableEditPolicy} bzw. {@link ResizableEditPolicy}
	 *         fr {@link DoModel}-Edit Parts, sonst <code>null</code>.
	 */
	@Override
	protected EditPolicy createChildEditPolicy(final EditPart child) {
		if (child instanceof DoModelEditPart<?, ?>) {
			child.installEditPolicy(EditPolicy.COMPONENT_ROLE,
					new DoModelEditPolicy());

			if (child.getModel() instanceof Sized) {
				return new ResizableEditPolicy();
			}
			return new NonResizableEditPolicy();
		}

		// fallback
		return null;
	}

}
