/**
 * 
 */
package de.bsvrz.buv.plugin.darstellung.editpolicies;

import org.eclipse.core.runtime.Platform;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.gef.EditPart;
import org.eclipse.gef.Request;
import org.eclipse.gef.commands.Command;
import org.eclipse.gef.editpolicies.ComponentEditPolicy;
import org.eclipse.gef.requests.ChangeBoundsRequest;
import org.eclipse.gef.requests.GroupRequest;

import de.bsvrz.buv.plugin.darstellung.commands.DeleteDoModelCommand;
import de.bsvrz.buv.plugin.darstellung.commands.MoveDoModelCommand;
import de.bsvrz.buv.plugin.dobj.HotspotConverter;
import de.bsvrz.buv.plugin.dobj.model.DoModel;

/**
 * {@link ComponentEditPolicy} die das Lschen eines {@link DoModel} steuert.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class DoModelEditPolicy extends ComponentEditPolicy {

	@Override
	protected Command createDeleteCommand(final GroupRequest deleteRequest) {
		final DoModel doModel = (DoModel) getHost().getModel();
		return new DeleteDoModelCommand(doModel);
	}

	@Override
	public Command getCommand(final Request request) {
		if (REQ_ORPHAN.equals(request.getType())
				&& request instanceof ChangeBoundsRequest) {
			return createMoveCommand((ChangeBoundsRequest) request);
		}

		return super.getCommand(request);
	}

	private Command createMoveCommand(
			final ChangeBoundsRequest request) {
		final EditPart editPart = (EditPart) request.getEditParts().get(0);
		final DoModel model = (DoModel) editPart.getModel();
		final Point location = getHotspotLocation(editPart,
				request.getLocation());
		return new MoveDoModelCommand(model, location);
	}

	private Point getHotspotLocation(final EditPart child,
			final Point boundsLocation) {
		HotspotConverter hotspotConverter = (HotspotConverter) child
				.getAdapter(HotspotConverter.class);
		if (hotspotConverter == null) {
			hotspotConverter = (HotspotConverter) Platform.getAdapterManager()
					.getAdapter(child, HotspotConverter.class);
		}

		Point location;
		if (hotspotConverter != null) {
			location = hotspotConverter.convertViewToModel(boundsLocation);
		} else {
			location = boundsLocation;
		}
		return location;
	}

}
