package de.bsvrz.buv.plugin.darstellung.wizards;

import java.io.File;
import java.io.IOException;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IExportWizard;
import org.eclipse.ui.PlatformUI;

import com.bitctrl.lib.eclipse.emf.eclipse.model.Named;
import com.bitctrl.lib.eclipse.emf.util.EmfUtil;

import de.bsvrz.buv.plugin.dobj.internal.DObjPlugin;

/**
 * Basisklasse fr einen Wizard, die ein in den Rahmenwerkseinstellungen
 * gespeichertes {@link Named}-Objekt exportiert.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public abstract class AbstractExportWizard extends AbstractWorkbenchWizard
		implements IExportWizard {

	private AbstractExportWizardPage exportPage;

	/**
	 * Initialisiert den Wizard.
	 * 
	 * @param eClass
	 *            die Klasse der zu exportierenden Objekte.
	 */
	protected AbstractExportWizard(final EClass eClass) {
		super(eClass);
	}

	/**
	 * Legt nur eine Wizard Page fr die Exporteinstellungen an.
	 * 
	 * @see #createExportPage()
	 */
	@Override
	public void addPages() {
		exportPage = createExportPage();
		exportPage.setPreSelection(getSelection());
		addPage(exportPage);
	}

	/**
	 * Erzeugt eine neue Wizard Page fr die Exporteinstellungen.
	 */
	protected abstract AbstractExportWizardPage createExportPage();

	@Override
	public boolean performFinish() {
		final String directory = exportPage.getDirectory();

		final ResourceSet resSet = new ResourceSetImpl();
		final IStructuredSelection export = (IStructuredSelection) exportPage
				.getSelection();
		for (final Object e : export.toList()) {
			if (!getEClass().isInstance(e)) {
				continue;
			}

			final Named object = (Named) e;
			final File file = new File(directory + "/" + object.getName() + "."
					+ getEClass().getName().toLowerCase());

			// Prfen ob die zu schreibende Datei bereits existiert
			if (file.exists()) {
				if (!MessageDialog.openQuestion(getShell(),
						"Datei berschreiben",
						"Es existiert bereits eine Datei mit dem selben Namen "
								+ file + ". Wollen Sie diese berschreiben?")) {
					return false;
				}

			}

			// Datei schreiben
			final Resource resource = resSet.createResource(URI.createURI(file
					.toURI().toString()));
			resource.getContents().add(object);
			try {
				resource.save(null);
			} catch (final IOException ex) {
				final String message = "Die Datei konnte nicht geschrieben werden: "
						+ EmfUtil.getText(object);
				final IStatus status = DObjPlugin.getDefault().getLogger()
						.error(message, ex);

				ErrorDialog.openError(PlatformUI.getWorkbench()
						.getActiveWorkbenchWindow().getShell(),
						"Fehler beim Export", message, status);

				return false;
			}
		}

		return true;
	}

}
