package de.bsvrz.buv.plugin.darstellung.wizards;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.INewWizard;

import de.bsvrz.buv.plugin.darstellung.model.Ansicht;
import de.bsvrz.buv.plugin.darstellung.model.DarstellungPackage;
import de.bsvrz.buv.plugin.darstellung.util.AnsichtenEinstellungen;
import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungenWaehler.EinstellungsArt;
import de.bsvrz.buv.rw.bitctrl.eclipse.wizards.EinstellungsArtWizardPage;

/**
 * Wizard zum Anlegen einer neuen Ansicht.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class AnsichtNewWizard extends AbstractWorkbenchWizard implements
		INewWizard {

	/** Konstante fr diesen Wizard. */
	public static final String WIZARD_ID = AnsichtNewWizard.class.getName();

	private AnsichtNewWizardPage ansichtPage;
	private EinstellungsArtWizardPage einstellungsArtPage;

	/**
	 * Initialisiert den Wizard.
	 */
	public AnsichtNewWizard() {
		super(DarstellungPackage.Literals.ANSICHT);
	}

	@Override
	public void addPages() {
		ansichtPage = new AnsichtNewWizardPage(AnsichtNewWizardPage.class
				.getName());
		addPage(ansichtPage);

		einstellungsArtPage = new EinstellungsArtWizardPage(
				EinstellungsArtWizardPage.class.getName());
		einstellungsArtPage.setMultiple(false);
		addPage(einstellungsArtPage);
	}

	@Override
	public boolean performFinish() {
		final Ansicht ansicht = ansichtPage.getAnsicht();
		final EinstellungsArt art = einstellungsArtPage
				.getFirstEinstellungsArt();

		if (AnsichtenEinstellungen.INSTANCE.getModellEinstellungen(art, ansicht
				.getName()) != null) {
			if (MessageDialog
					.openQuestion(
							getShell(),
							"Ansicht  berschreiben",
							"Eine Ansicht mit dem Namen \""
									+ ansicht.getName()
									+ "\" existiert bereits in den Einstellungen \""
									+ art
									+ "\".\n\nWollen Sie diese Ansicht berschreiben? Diese Aktion kann nicht rckgngig gemacht werden.")) {
				AnsichtenEinstellungen.INSTANCE.setModellEinstellungen(art,
						ansicht.getName(), ansicht);
			} else {
				return false;
			}
		} else {
			AnsichtenEinstellungen.INSTANCE.setModellEinstellungen(art, ansicht
					.getName(), ansicht);
		}

		return true;
	}

}
