package de.bsvrz.buv.plugin.dobj.actions;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.gef.EditPart;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.actions.ActionFactory;

import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.util.DobjUtil;
import de.bsvrz.buv.plugin.dobj.util.SelectionProperties;

/**
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public class SelectAllAction extends DobjWorkbenchPartAction {

	/**
	 * Initialisiert die Action.
	 * 
	 * @param part
	 *            der Workbench Part in dem alle Objekte selektiert werden
	 *            sollen.
	 */
	public SelectAllAction(final IWorkbenchPart part) {
		super(part);
		setText("Alles markieren");
		setId(ActionFactory.SELECT_ALL.getCommandId());
		setActionDefinitionId(ActionFactory.SELECT_ALL.getCommandId());
	}

	@Override
	public void run() {
		final List<EditPart> selList = new ArrayList<EditPart>();
		for (final EditPart ep : DobjUtil.getAllChildren(getViewer()
				.getContents())) {
			if (ep.isSelectable() && isValidModel(ep.getModel())) {
				selList.add(ep);
			}
		}
		getViewer().setSelection(new StructuredSelection(selList));
	}

	/**
	 * Prft das Modellobjekt eines Edit Parts. Es ist valid, wenn es ein
	 * {@link DoModel} ist und wenn {@link SelectionProperties} vorhanden sind,
	 * muss es in der Matchingmenge enthalten sein.
	 */
	protected boolean isValidModel(final Object object) {
		final SelectionProperties selectionProperties = (SelectionProperties) getViewer()
				.getProperty(SelectionProperties.class.toString());

		if (object instanceof DoModel) {
			if (selectionProperties == null) {
				return true;
			}

			final DoModel model = (DoModel) object;
			return selectionProperties.getMatchDoTypen().contains(
					model.getDoTyp());
		}

		return false;
	}

}
