package de.bsvrz.buv.plugin.dobj.actions;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.Request;
import org.eclipse.jface.action.IAction;
import org.eclipse.ui.IWorkbenchPart;

import de.bsvrz.buv.plugin.dobj.internal.DobjMessages;
import de.bsvrz.buv.plugin.dobj.requests.BaustelleGueltigRequest;
import de.bsvrz.buv.plugin.dobj.util.DobjIcons;

/**
 * Action zum Ein- und Auschalten des Filters fuer <b>alle</b> gueltige
 * Baustellen.
 * 
 * <p>
 * Der Workbench Part muss einen {@link GraphicalViewer} besitzen. An die Edit
 * Parts des Viewers wird entsprechend der Auswahl des Benutzer ein
 * {@link BaustelleGueltigRequest} gesendet.
 * 
 * @author BitCtrl Systems GmbH, Thomas Thierfelder
 * @version $Id$
 */
public class ToggleBaustelleGueltigAction extends DobjWorkbenchPartAction {

	/**
	 * Initialisiert die Action.
	 * 
	 * @param part
	 *            der Workbench Part mit dem die Action arbeiten soll.
	 */
	public ToggleBaustelleGueltigAction(final IWorkbenchPart part) {
		super(part, IAction.AS_CHECK_BOX);

		setText(DobjMessages.ToggleBaustelleGueltig_Label);
		setToolTipText(DobjMessages.ToggleBaustelleGueltig_Tooltip);
		setId(DobjActionFactory.TOGGLE_BAUSTELLE_GUELTIG.getCommandId());
		setActionDefinitionId(DobjActionFactory.TOGGLE_BAUSTELLE_GUELTIG
				.getCommandId());
		setImageDescriptor(DobjIcons.ActionToggleBaustelleGueltig
				.getImageDescriptor());
		setChecked(true);
	}

	@Override
	public void run() {
		final GraphicalViewer viewer = getViewer();

		final Request req = new BaustelleGueltigRequest(isChecked());
		for (final Object e : viewer.getEditPartRegistry().values()) {
			final EditPart editPart = (EditPart) e;
			editPart.performRequest(req);
		}
	}

}
