package de.bsvrz.buv.plugin.dobj.actions;

import org.eclipse.gef.EditPart;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.Request;
import org.eclipse.jface.action.IAction;
import org.eclipse.ui.IWorkbenchPart;

import de.bsvrz.buv.plugin.dobj.internal.DobjMessages;
import de.bsvrz.buv.plugin.dobj.requests.VerbindungslinieRequest;
import de.bsvrz.buv.plugin.dobj.util.DobjIcons;

/**
 * Action zum Ein- und Auschalten der Verbindungslinie eines verschobenen
 * Objekts zu seiner Originalposition.
 * 
 * <p>
 * Der Workbench Part muss einen {@link GraphicalViewer} besitzen. An die Edit
 * Parts des Viewers wird entsprechend der Auswahl des Benutzer ein
 * {@link VerbindungslinieRequest} gesendet.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id: ToggleVerbindungslinieAction.java 26063 2010-09-02 23:40:58Z
 *          Schumann $
 */
public class ToggleVerbindungslinieAction extends DobjWorkbenchPartAction {

	/**
	 * Initialisiert die Action.
	 * 
	 * @param part
	 *            der Workbench Part mit dem die Action arbeiten soll.
	 */
	public ToggleVerbindungslinieAction(final IWorkbenchPart part) {
		super(part, IAction.AS_CHECK_BOX);

		setText(DobjMessages.ToggleVerbindungslinie_Label);
		setToolTipText(DobjMessages.ToggleVerbindungslinie_Tooltip);
		setId(DobjActionFactory.TOGGLE_VERBINDUNGSLINIE.getCommandId());
		setActionDefinitionId(DobjActionFactory.TOGGLE_VERBINDUNGSLINIE
				.getCommandId());
		setImageDescriptor(DobjIcons.ActionToggleVerbindingslinie
				.getImageDescriptor());
		setChecked(true);
	}

	@Override
	public void run() {
		final GraphicalViewer viewer = getViewer();

		final Request req = new VerbindungslinieRequest(isChecked());
		for (final Object e : viewer.getEditPartRegistry().values()) {
			final EditPart editPart = (EditPart) e;
			editPart.performRequest(req);
		}
	}

}
