package de.bsvrz.buv.plugin.dobj.editparts;

import org.eclipse.draw2d.IFigure;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.jface.resource.ResourceManager;

import com.bitctrl.lib.eclipse.emf.gef.editparts.AdapterGraphicalEditPart;

import de.bsvrz.buv.plugin.dobj.internal.DObjPlugin;
import de.bsvrz.buv.plugin.dobj.model.DoModel;
import de.bsvrz.buv.plugin.dobj.util.DobjUtil;
import de.bsvrz.sys.funclib.bitctrl.daf.DavProvider;

/**
 * Basisklasse fr graphische Edit Parts, gleich ob sie ein {@link DoModel}
 * kontrollieren soll oder andere Modellobjekte.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 * @param <T>
 *            der Typ des Modells.
 * @param <F>
 *            der Typ der Figure.
 */
public abstract class BaseGraphicalEditPart<T extends EObject, F extends IFigure>
		extends AdapterGraphicalEditPart<T, F> {

	/**
	 * Gibt den Zoommanager des Edit Parts zurck. An ihn kann die aktuelle
	 * Zoomstufe erfragt und gesetzt oder auf nderungen angemeldet werden.
	 * 
	 * @return der Zoommanager des Edit Parts.
	 */
	public ZoomManager getZoomManager() {
		return (ZoomManager) getViewer().getProperty(
				ZoomManager.class.toString());
	}

	/**
	 * Gibt den Resource Manager des Viewers zurck, in dem der Edit Part
	 * dargestellt wird. Die damit angelegten Ressourcen werden wieder
	 * freigegeben, wenn der Viewer zerstrt wird.
	 * 
	 * @return ein Resource Manager.
	 */
	protected ResourceManager getResourceManager() {
		return getViewer().getResourceManager();
	}

	/**
	 * Gibt die Laufzeiteigenschaften der Darstellung zurck. Die wichtigsten
	 * Eigenschaften sind <code>editor</code> und <code>online</code>. *
	 * 
	 * @return <code>true</code>, wenn die Property {@link DobjUtil#PROP_EDITOR}
	 *         existiert und <code>true</code> ist, sonst <code>false</code>.
	 */
	protected boolean isEditor() {
		final Object result = getViewer().getProperty(DobjUtil.PROP_EDITOR);
		if (result instanceof Boolean) {
			return (Boolean) result;
		}

		return false;
	}

	/**
	 * Gibt den Typ des Editors zurck, in dem der EditPart verwendet wird. Die
	 * Standarddefinitionen sind in {@link DobjUtil} zu finden.
	 * 
	 * Wenn kein Typ deifniert ist wird 0 geliefert.
	 * 
	 * @return den Typ oder 0.
	 */
	protected int getEditorType() {
		final Object result = getViewer()
				.getProperty(DobjUtil.PROP_EDITOR_TYPE);
		if (result instanceof Integer) {
			return (Integer) result;
		}

		return 0;
	}

	/**
	 * Gibt die Nutzerverbindung zum Datenverteiler zurck.
	 * 
	 * @return ein Provider, der die Datenverteilerverbindung bereitstellt.
	 */
	protected DavProvider getDavProvider() {
		return DObjPlugin.getDefault().getDavProvider();
	}

}
