package de.bsvrz.buv.plugin.dobj.editparts;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import org.eclipse.draw2d.IFigure;

import de.bsvrz.buv.plugin.dobj.util.Projektion;
import de.bsvrz.sys.funclib.bitctrl.modell.tmgeoreferenzierungglobal.objekte.GeoReferenzObjekt;

/**
 * Basisklasse fr Systemobjekte vom typ {@link GeoReferenzObjekt}.
 * 
 * <p>
 * Diese Klasse ist nur dafr da, um auf nderung der {@link Projektion}
 * anzumelden. ndert sich die Projektion, dann wird
 * {@link #cleanModellKoordinaten()} aufgerufen. Wie und welche Koordinaten
 * bestimmt werden wird in den Unterklassen bestimmt.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 * @param <T>
 * @param <F>
 * @see PunktEditPart
 * @see LinieEditPart
 * @see FlaecheEditPart
 * @see KomplexEditPart
 */
public abstract class GeoRefererenzObjektEditPart<T extends GeoReferenzObjekt, F extends IFigure>
		extends BitCtrlDoModelEditPart<T, F> {

	private final PropertyChangeListener propertyListener = new PropertyChangeListener() {

		@Override
		public void propertyChange(final PropertyChangeEvent evt) {
			final String prop = evt.getPropertyName();

			if (Projektion.class.toString().equals(prop)) {
				cleanModellKoordinaten();
				refreshVisuals();
			}
		}

	};

	/**
	 * Entfernt alle gecachten Modellkoordinaten (Pixelkoordinaten), damit diese
	 * neu berechnet werden.
	 */
	protected abstract void cleanModellKoordinaten();

	@Override
	public void activate() {
		super.activate();

		if (isEditor()) {
			getViewer().addPropertyChangeListener(propertyListener);
		}
	}

	@Override
	public void deactivate() {
		if (isEditor()) {
			getViewer().removePropertyChangeListener(propertyListener);
		}

		super.deactivate();
	}

}
