package de.bsvrz.buv.plugin.dobj.editparts;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.draw2d.geometry.PrecisionPoint;

import de.bsvrz.sys.funclib.bitctrl.geolib.WGS84Koordinate;
import de.bsvrz.sys.funclib.bitctrl.modell.tmgeoreferenzierungglobal.konfigurationsdaten.KdKomplexKoordinaten;
import de.bsvrz.sys.funclib.bitctrl.modell.tmgeoreferenzierungglobal.objekte.Flaeche;
import de.bsvrz.sys.funclib.bitctrl.modell.tmgeoreferenzierungglobal.objekte.Komplex;
import de.bsvrz.sys.funclib.bitctrl.modell.tmgeoreferenzierungglobal.objekte.KomplexXY;
import de.bsvrz.sys.funclib.bitctrl.modell.util.geolib.WGS84Util;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Basisklasse fr Edit Parts die als Controller fuer Systemobjekte die
 * <code>typ.komplex</code> erweitern.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 * @param <T>
 */
public abstract class KomplexEditPart<T extends Komplex, F extends IFigure>
		extends GeoRefererenzObjektEditPart<T, F> {

	// TODO Punkt-, Linien- und Komplexobjekte einbeziehen

	private List<PointList> flaechen;

	@Override
	protected void cleanModellKoordinaten() {
		flaechen = null;
	}

	/**
	 * Bestimmt die in dem Komplex-Objekt enthaltenen Flchen.
	 */
	protected List<PointList> getFlaechen() {
		if (flaechen == null) {
			final KomplexXY komplex = (KomplexXY) getSystemObjekt();
			try {
				flaechen = new ArrayList<PointList>();
				final KdKomplexKoordinaten.Daten datum = komplex
						.getKdKomplexKoordinaten().getDatum();
				if (datum != null) {
					for (final Flaeche flaeche : datum.getFlaechenReferenz()) {
						final PointList points = new PointList();
						for (final WGS84Koordinate k : WGS84Util
								.konvertiere(flaeche)) {
							final Point p = getProjektion()
									.umrechnenWeltNachModel(
											new PrecisionPoint(k.getLaenge(), k
													.getBreite()));
							points.addPoint(p);
						}
						flaechen.add(points);
					}
				}
			} catch (final RuntimeException ex) {
				Debug.getLogger().error(
						"Die WGS-84-Koordinaten des Komplex konnten nicht bestimmt werden: "
								+ komplex, ex);
			}
		}
		return flaechen;
	}

}
