package de.bsvrz.buv.plugin.dobj.editpolicies;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Cursors;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.PointList;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.GraphicalEditPart;
import org.eclipse.gef.handles.AbstractHandle;
import org.eclipse.gef.handles.MoveHandleLocator;
import org.eclipse.gef.handles.NonResizableHandleKit;
import org.eclipse.gef.tools.SelectEditPartTracker;

import com.bitctrl.lib.eclipse.gef.editpolicies.OnlySelectionEditPolicy;

import de.bsvrz.buv.plugin.dobj.editparts.DoModelEditPart;
import de.bsvrz.buv.plugin.dobj.editparts.LinieEditPart;

/**
 * Seletionspolicy, die allen Linien (Editparts vom Typ {@link LinieEditPart})
 * ein Selektionshandle zuordnet, das eine rote Linie ueber der referenzierten
 * Linie als Markierung zeichnet.
 * 
 * @author BitCtrl Systems GmbH, thierfelder
 * 
 * @version $Id$
 */
public class MqAlsLinieSelectionEditPolicy extends OnlySelectionEditPolicy {

	@Override
	protected List<?> createSelectionHandles() {
		final List<Object> list = new ArrayList<Object>();

		if (getHost() != null && getHost() instanceof LinieEditPart) {
			final LinieEditPart<?, ?> lep = (LinieEditPart<?, ?>) getHost();
			list.add(new LinieHandle(lep));
		} else {
			NonResizableHandleKit.addHandles((GraphicalEditPart) getHost(),
					list, new SelectEditPartTracker(getHost()), Cursors.ARROW);
		}

		return list;
	}

	/**
	 * Handle zur Markierunng von Strassensegmenten bzw. Linien mit einem roten
	 * Strich.
	 * 
	 * @author BitCtrl Systems GmbH, thierfelder
	 * 
	 * @version $Id$
	 */
	private static class LinieHandle extends AbstractHandle {

		private LinieHandle(final LinieEditPart<?, ?> owner) {
			setOwner(owner);
			setLocator(new MoveHandleLocator(owner.getFigure()));
			setCursor(Cursors.ARROW);
		}

		@Override
		public void paintFigure(final Graphics g) {
			if (getOwner() != null && getOwner().getFigure() != null
					&& getOwner() instanceof DoModelEditPart<?, ?>) {
				final double zoom = ((DoModelEditPart<?, ?>) getOwner())
						.getZoomManager().getZoom();
				final PointList originalPoints = ((LinieEditPart<?, ?>) getOwner())
						.getPoints();
				final PointList scaledPoints = new PointList();
				for (int i = 0; i < originalPoints.size(); i++) {
					scaledPoints.addPoint(new Point(
							originalPoints.getPoint(i).x * zoom, originalPoints
									.getPoint(i).y * zoom));
				}
				g.setLineWidth(3);
				g.setForegroundColor(ColorConstants.red);
				g.drawPolyline(scaledPoints);
			}
		}

		@Override
		protected DragTracker createDragTracker() {
			return new SelectEditPartTracker(getOwner());
		}

	}

}
