/**
 * 
 */
package de.bsvrz.buv.plugin.dobj.figures;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.eclipse.draw2d.BorderLayout;
import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.ImageFigure;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.graphics.ImageData;

import com.bitctrl.lib.eclipse.draw2d.svg.SVGCache;
import com.bitctrl.lib.eclipse.draw2d.svg.SVGFigure;

import de.bsvrz.buv.plugin.dobj.internal.DObjPlugin;
import de.bsvrz.buv.plugin.dobj.util.BildManager;
import de.bsvrz.sys.funclib.bitctrl.modell.att.Feld;
import de.bsvrz.sys.funclib.bitctrl.modell.bitctrlallgemein.objekte.Bild;
import de.bsvrz.sys.funclib.bitctrl.modell.metamodellglobal.attribute.AttByte_JavaKeyword;

/**
 * Bild - Figur, die sowohl SVG, als auch normal binary Bilder (png,bmp,jpg)
 * darstellen kann.
 * 
 * @author BitCtrl Systems GmbH, Hoesel
 * @version $Id$
 */
public class DoBildFigure extends Figure {

	private Bild bild;
	private Image image;
	private ImageFigure imgFigure;
	private SVGFigure svgFigure;

	/**
	 * Konstruktor fr eine Bild - Figur, die sowohl SVG, als auch normal binary
	 * Bilder (png,bmp,jpg) darstellen kann.
	 */
	public DoBildFigure() {
		super();
		final BorderLayout layout = new BorderLayout();
		setLayoutManager(layout);
	}

	/**
	 * @return the bild
	 */
	public final Bild getBild() {
		return bild;
	}

	/**
	 * Erneutes skalieren und "zeichnen" des Bildes auf eine interne
	 * {@link Figure}.
	 */
	private void refreshBild() {

		if (image != null && !image.isDisposed()) {
			image.dispose();
			image = null;
		}

		if (bild != null) {
			if ("image/svg+xml".equalsIgnoreCase(bild.getKdBild().getDatum()
					.getMIMEType())) {

				final Feld<AttByte_JavaKeyword> bilddaten = bild.getKdBild()
						.getDatum().getBilddaten();
				// TODO:u.U. kann man hier das Array noch Cachen
				final byte[] buffer = new byte[bilddaten.size()];
				for (int i = 0; i < bilddaten.size(); ++i) {
					buffer[i] = bilddaten.get(i).byteValue();
				}

				final File file;
				FileOutputStream fout = null;
				try {
					file = DObjPlugin
							.getDefault()
							.getBundle()
							.getBundleContext()
							.getDataFile(
									bild.getPid() + System.currentTimeMillis()
											+ ".tmp");
					fout = new FileOutputStream(file);
					fout.write(buffer);
					fout.flush();
					fout.close();
					final SVGCache svg = new SVGCache();
					svg.setURI(file.toURI().toString(), true);
					removeAll();
					imgFigure = null;
					svgFigure = new SVGFigure();
					svgFigure.setSvgCache(svg);
					add(svgFigure, BorderLayout.CENTER);

					file.delete();
				} catch (final IOException ex) {
					Logger.getLogger(getClass().getName())
							.log(Level.SEVERE,
									"Laden eines SVG Hintergrundbildes ist fehlgeschlagen.",
									ex);
				}
			} else {

				final ImageDescriptor desc = BildManager.INSTANCE
						.getImageDescriptor(bild);

				if (!getClientArea().isEmpty()) {
					final ImageData data = desc.getImageData().scaledTo(
							getClientArea().width, getClientArea().height);
					image = ImageDescriptor.createFromImageData(data)
							.createImage();
				} else {
					image = ImageDescriptor.createFromImageData(
							desc.getImageData()).createImage();
				}

				removeAll();
				svgFigure = null;
				imgFigure = new ImageFigure(image);
				add(imgFigure, BorderLayout.CENTER);
			}
		}
	}

	/**
	 * @param bild
	 *            the bild to set
	 */
	public final void setBild(final Bild bild) {
		if (this.bild != null && this.bild.equals(bild) || this.bild == null
				&& bild == null) {
			return;
		}
		this.bild = bild;
		refreshBild();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.draw2d.Figure#setSize(int, int)
	 */
	@Override
	public void setSize(final int w, final int h) {
		super.setSize(w, h);
		refreshBild();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.draw2d.ImageFigure#paintFigure(org.eclipse.draw2d.Graphics)
	 */
	@Override
	protected void paintFigure(final Graphics graphics) {
		// if (bild != null && (svgFigure == null || imgFigure == null)) {
		// setBild(bild);
		// }
		super.paintFigure(graphics);
	}

}
