/*
 * Rahmenwerk-Plug-in "Mastbliche Darstellung"
 * Copyright (C) 2009 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.dobj.kollision.travers;

import org.eclipse.draw2d.geometry.Point;

import de.bsvrz.buv.plugin.dobj.vektor.Vektor2D;

/**
 * Sucht einen Kegel mit der Spitze in einem Punkt ab. Begonnen wird dabei mit
 * der Spitze des Kegels und von dieser aus gesehen zum Stumpf des Kegels. Auf
 * jeder Hoehe wird sich jeweils abwechselnd nach links und rechts mit
 * zunehmendem Abstand zur Kegelachse bewegt.<br>
 * Die Punkte des Kegels werden so abgesucht, dass der Abstand zwischen ihnen
 * maximal ist.
 * 
 * @author BitCtrl Systems GmbH, thierfelder
 * @version $Id$
 */
public class KegelUeberPunktAbsucher extends PunktTraverser {

	/**
	 * Standardkonstruktor.
	 * 
	 * @param startMittelPunkt
	 *            Spitze des Kegels und Startpunkt der Suche.
	 * @param normierterNormalenVektor
	 *            normierter Normalenvektor im Startpunkt des Kegels zum Stumpf
	 *            hin.
	 * @param achsenLaenge
	 *            Achsenlaenge des Kegels.
	 * @param anzahlPunkteSoll
	 *            Anzahl der Punkte, die innerhalb des Kegels besucht werden
	 *            sollen.
	 */
	public KegelUeberPunktAbsucher(final Point startMittelPunkt,
			final Vektor2D normierterNormalenVektor, final double achsenLaenge,
			final int anzahlPunkteSoll) {
		super(startMittelPunkt);
		final double radius = achsenLaenge / 2.0;
		final double step = achsenLaenge
				/ Math.sqrt(2.0 * Math.min(anzahlPunkteSoll, achsenLaenge
						* achsenLaenge));

		final Vektor2D antennaLinks = normierterNormalenVektor
				.getNormalenVektorLinks();
		final Vektor2D antennaRechts = normierterNormalenVektor
				.getNormalenVektorRechts();
		double inset = radius;
		for (double laengeSenkrechte = 0; laengeSenkrechte < achsenLaenge; laengeSenkrechte += step) {
			for (double laengeWaagerechte = 0; laengeWaagerechte < radius
					- inset; laengeWaagerechte += step) {
				final Point punktLinks = new Point(startMittelPunkt.x
						+ laengeSenkrechte * normierterNormalenVektor.x
						+ antennaLinks.x * laengeWaagerechte,
						startMittelPunkt.y + laengeSenkrechte
								* normierterNormalenVektor.y + antennaLinks.y
								* laengeWaagerechte);
				final Point punktRechts = new Point(startMittelPunkt.x
						+ laengeSenkrechte * normierterNormalenVektor.x
						+ antennaRechts.x * laengeWaagerechte,
						startMittelPunkt.y + laengeSenkrechte
								* normierterNormalenVektor.y + antennaRechts.y
								* laengeWaagerechte);

				punktListe.add(punktLinks);
				if (!punktLinks.equals(punktRechts)) {
					punktListe.add(punktRechts);
				}
			}
			inset -= step;
		}
	}
}
