package de.bsvrz.buv.plugin.dobj.tools;


/**
 * Kann als Basisklasse dienen fr Tools die etwas per Drag tun. Der Vorgang
 * beginnt, wenn die linke Maustaste gedrck wird und endet, wenn die Maustaste
 * wieder losgelassen wird. Whrend die Maustaste gedrckt ist, wird ein
 * Feedback angezeigt, welches bei Bewegung der Maus aktualisiert wird.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public abstract class AbstractDragTool extends BaseTool {

	@Override
	protected boolean handleDragInProgress() {
		if (isInState(STATE_DRAG | STATE_DRAG_IN_PROGRESS)) {
			showFeedback();
		}
		return true;
	}

	@Override
	protected boolean handleButtonDown(final int button) {
		if (!isGraphicalViewer()) {
			return true;
		}
		if (button != 1) {
			setState(STATE_INVALID);
			handleInvalidInput();
		}
		stateTransition(STATE_INITIAL, STATE_DRAG_IN_PROGRESS);
		return true;
	}

	@Override
	protected boolean handleButtonUp(final int button) {
		if (stateTransition(STATE_DRAG_IN_PROGRESS, STATE_TERMINAL)) {
			eraseFeedback();
		}

		if (button == 1) {
			handleFinished();
		}
		return true;
	}

	@Override
	protected boolean handleInvalidInput() {
		eraseFeedback();
		return true;
	}

	@Override
	public void deactivate() {
		if (isInState(STATE_DRAG_IN_PROGRESS)) {
			eraseFeedback();
		}
		super.deactivate();
		setState(STATE_TERMINAL);
	}

	/**
	 * Wird aufgerufen, wenn die Feedback Figure angezeigt werden soll. Sollte
	 * {@link #addFeedback(org.eclipse.draw2d.IFigure)} aufrufen.
	 */
	protected abstract void showFeedback();

	/**
	 * Wird aufgerufen, wenn die Feedback Figure wieder entfernt werden soll.
	 * Sollte {@link #removeFeedback(org.eclipse.draw2d.IFigure)} aufrufen.
	 * 
	 * @see #removeFeedback(org.eclipse.draw2d.IFigure)
	 */
	protected abstract void eraseFeedback();

}
