package de.bsvrz.buv.plugin.dobj.tools;

import org.eclipse.gef.EditPartViewer;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.Tool;
import org.eclipse.gef.editparts.ZoomManager;
import org.eclipse.gef.tools.AbstractTool;

/**
 * Kann als Basisklasse fr {@link Tool}s dienen, die fr einen
 * {@link GraphicalViewer} gedacht sind.
 * 
 * @author BitCtrl Systems GmbH, Falko Schumann
 * @version $Id$
 */
public abstract class BaseTool extends AbstractTool {

	/**
	 * Gibt den {@link ZoomManager} des {@link EditPartViewer} zurck.
	 * 
	 * @return der Zoom Manager oder <code>null</code>, wenn es keinen gibt.
	 */
	protected ZoomManager getZoomManager() {
		return (ZoomManager) getCurrentViewer().getProperty(
				ZoomManager.class.toString());
	}

	/**
	 * Prft, ob der aktuelle {@link EditPartViewer} ein {@link GraphicalViewer}
	 * ist.
	 */
	protected boolean isGraphicalViewer() {
		return getCurrentViewer() instanceof GraphicalViewer;
	}

	/**
	 * Prft, ob der bergebene Viewer ein {@link GraphicalViewer} ist.
	 */
	@Override
	protected boolean isViewerImportant(final EditPartViewer viewer) {
		return viewer instanceof GraphicalViewer;
	}

}