package de.bsvrz.buv.plugin.dobj.viewers;

import java.util.Arrays;
import java.util.Collection;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.Viewer;

import com.bitctrl.lib.eclipse.emf.util.EmfComparator;

import de.bsvrz.buv.plugin.dobj.model.DoTyp;
import de.bsvrz.buv.plugin.dobj.util.DoTypenUtil;
import de.bsvrz.dav.daf.main.config.SystemObjectType;
import de.bsvrz.sys.funclib.bitctrl.util.SystemObjectComparator;

public class DoTypenTreeContentProvider implements ITreeContentProvider {

	private SortedMap<SystemObjectType, SortedSet<DoTyp>> mappings;

	@Override
	public Object[] getElements(final Object inputElement) {
		mappings = new TreeMap<SystemObjectType, SortedSet<DoTyp>>(
				new SystemObjectComparator());

		Collection<DoTyp> doTypen;
		if (inputElement instanceof Collection) {
			doTypen = (Collection<DoTyp>) inputElement;
		} else if (inputElement instanceof Object[]) {
			doTypen = Arrays.asList((DoTyp[]) inputElement);
		} else {
			doTypen = DoTypenUtil.getDoTypen();
		}

		for (final DoTyp doTyp : doTypen) {
			final SystemObjectType type = doTyp.getSystemObjectType();
			if (type != null) {
				if (!mappings.containsKey(type)) {
					mappings.put(type, new TreeSet<DoTyp>(new EmfComparator()));
				}
				mappings.get(type).add(doTyp);
			}
		}

		return mappings.keySet().toArray();
	}

	@Override
	public boolean hasChildren(final Object element) {
		if (element instanceof SystemObjectType) {
			return mappings.containsKey(element)
					&& !mappings.get(element).isEmpty();
		}

		return false;
	}

	@Override
	public Object[] getChildren(final Object parentElement) {
		if (parentElement instanceof SystemObjectType) {
			return mappings.get(parentElement).toArray();
		}

		return new Object[0];
	}

	@Override
	public Object getParent(final Object element) {
		if (element instanceof DoTyp) {
			return ((DoTyp) element).getSystemObjectType();
		}
		return null;
	}

	@Override
	public void inputChanged(final Viewer viewer, final Object oldInput,
			final Object newInput) {
		// tut nix
	}

	@Override
	public void dispose() {
		// tut nix
	}

}