/*
 * Rahmenwerk-Plug-in "Darstellungsobjekttyp-Editor"
 * Copyright (C) 2010 BitCtrl Systems GmbH 
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 51
 * Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
 *
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weissenfelser Strasse 67
 * 04229 Leipzig
 * Phone: +49 341-490670
 * mailto: info@bitctrl.de
 */

package de.bsvrz.buv.plugin.doeditor.figures;

import org.eclipse.draw2d.Figure;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.LayoutListener;
import org.eclipse.draw2d.XYLayout;

import de.bsvrz.buv.plugin.doeditor.editparts.DecoratorDatenSatz;
import de.bsvrz.buv.plugin.doeditor.model.ExternalDoForm;
import de.bsvrz.buv.plugin.doeditor.model.ParameterDefinition;
import de.bsvrz.buv.plugin.doeditor.model.SichtbarkeitParameterDefinition;

/**
 * Figur zur Darstellung einer eingebetten Instanz eines Darstellungsobjekttyps,
 * der programmiert und ber einen Extension-Point eingebunden wurde.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 * @version $Id: ExternalDoFormFigure.java 34123 2012-05-21 12:19:09Z schumann $
 */
public class ExternalDoFormFigure extends Figure implements VisibleFormFigure {

	/** das Modellobjekt, das das eingebette Element reprsentiert. */
	private final ExternalDoForm externalDo;

	/**
	 * Konstruktor.
	 * 
	 * @param externalDo
	 *            das Modellobjekt
	 */
	public ExternalDoFormFigure(final ExternalDoForm externalDo) {
		this.externalDo = externalDo;
		setLayoutManager(new XYLayout());

		/*
		 * der Layoutmanager wird verwendet, um die Gre des Elements zu
		 * aktualisieren, wenn alle untergeordneten Formen eingefgt wurden.
		 */
		addLayoutListener(new LayoutListener() {

			@Override
			public void invalidate(final IFigure container) {
				// keine Aktionen erforderlich
			}

			@Override
			public boolean layout(final IFigure container) {
				return false;
			}

			@Override
			public void postLayout(final IFigure container) {
				// keine Aktionen erforderlich
			}

			@Override
			public void remove(final IFigure child) {
				// wird hier nicht ausgefhrt
			}

			@Override
			public void setConstraint(final IFigure child,
					final Object constraint) {
				setSize(child.getSize());
			}
		});

	}

	@Override
	public void aktualisiereVomModel() {
		setLocation(externalDo.getLocation());
		setVisible(externalDo.isVisible());
	}

	@Override
	public void decorate(final ParameterDefinition decorator,
			final DecoratorDatenSatz datenSatz) {
		if (decorator instanceof SichtbarkeitParameterDefinition) {
			setVisible(((SichtbarkeitParameterDefinition) decorator)
					.isSichtbar());
		}
	}

	@Override
	public double getScale() {
		/*
		 * die Bercksichtigung Skalierung wird fr dieses Element nicht
		 * untersttzt
		 */
		return 1.0d;
	}

	@Override
	public void setScale(final double scale) {
		/*
		 * die Bercksichtigung Skalierung wird fr dieses Element nicht
		 * untersttzt
		 */
	}

	@Override
	protected boolean useLocalCoordinates() {
		return true;
	}

}
